<?php

namespace Teknisa\CRM\Factory;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class AgrupamentoParceiroFactory
{

    protected $connection;

    public function __construct(EntityManager $entityManager)
    {

        $this->connection = $entityManager->getConnection();
    }

    public function queryDeleteGroup($paramsQuery)
    {
        try {
            $delete = "DELETE FROM CRM_AGRUPAMENTOPARCEIRO
                        WHERE NRAGRPARC = :NRAGRPARC
                        AND CRM_NRORGTRAB = :CRM_NRORGTRAB
                        AND NRORG = :NRORG";

            $std = $this->connection->prepare($delete);
            $std->execute($paramsQuery);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function verifyGroup($paramsQuery)
    {
        try {
            $query = "SELECT 
                         NRAGRPARC, 
                         NMPRINCIPALPARC,
                         NRINSCRICAOPARC
                       FROM 
                         CRM_AGRUPAMENTOPARCEIRO AGR
                         LEFT JOIN PARCNEGOCIO PARC
                           ON PARC.NRPARCNEGOCIO = AGR.NRPARCNEGOCIO
                        WHERE 
                          AGR.NRPARCNEGOCIO_REL = :NRPARCNEGOCIO_REL
                          AND AGR.CRM_NRORGTRAB = :CRM_NRORGTRAB
                          AND AGR.NRORG = :NRORG";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);

            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp[0];
            } else {
                return $resp;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insertTaskRel($paramsQuery)
    {
        try {
            $query = "INSERT INTO FILACONTATO (NRSEQFILACONTA, NRSEQCONTATO, DSFILACONTATO, DTPLANFILACONTA, CDOPERREAL,
                NRPARCNEGOCIO, IDORIGFILACONTA,NRPRIORFILACONT, NRORG, CRM_NRORGTRAB,NRSEQCAMPANHA)
            VALUES (:NRSEQFILACONTA,:NRSEQCONTATO,:DSFILACONTATO,:DTPLANFILACONTA,:CDOPERREAL,
                 :NRPARCNEGOCIO,:IDORIGFILACONTA,:NRPRIORFILACONT, :NRORG, :CRM_NRORGTRAB,:NRSEQCAMPANHA)";
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
        } catch (\Exception $e) {
            $e->getMessage();
            throw new \Exception($e);
        }
    }

    public function agrupamentoEvolSave($paramsQuery)
    {
        try {
            $query = "INSERT INTO 
                      EVOLCONT(NRCONTATO, DTHREVOLCONT, CDTIPOCONT, IDSENEVOCONT, IDTPEVOLCONT, CDOPEREVOL, CDMOTIEVOCON, NRORG, CRM_NRORGTRAB, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, TXEVOLCONT, IDATIVO, NRSEQRESULEVOL, NRSEQEVOLCONT, NRSEQCONTATO, IDGERAFATURA, IDANEXOEVOLCONT, CDFILIAL)
                      VALUES(:NRCONTATO, :DTHREVOLCONT, :CDTIPOCONT, :IDSENEVOCONT, 'P', :CDOPEREVOL, :CDMOTIEVOCON, :NRORG, :CRM_NRORGTRAB, SYSDATE, SYSDATE, :NRORG, :CDOPEREVOL, :NRORG, :CDOPEREVOL, :TXEVOLCONT, 'S', :NRSEQRESULEVOL, :NRSEQEVOLCONT, :NRSEQCONTATO, 'NAO', 'N', :CDFILIAL)";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
        } catch (\Exception $e) {
            $e->getMessage();
            throw new \Exception($e);
        }
    }

    public function searchNrContato($paramsQuery)
    {
        try {
            $query = "SELECT 
                         NRCONTATO, 
                         NRSEQCONTATO 
                       FROM 
                         CONTATO
                        WHERE 
                          NRPARCNEGOCIO = :NRPARCNEGOCIO
                          AND NRORG = :NRORG";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);

            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp[0];
            } else {
                return array();
            }
        } catch (\Exception $e) {
            $e->getMessage();
        }
    }

    public function verifyParcGroupRel($paramsQuery)
    {
        try {
            $query = "SELECT
                         AGR.NRAGRPARC, 
                         PARC.NRPARCNEGOCIO,
                         PARC.NMPRINCIPALPARC,
                         PARC.NRINSCRICAOPARC
                       FROM 
                         CRM_AGRUPAMENTOPARCEIRO AGR
                         LEFT JOIN PARCNEGOCIO PARC
                           ON PARC.NRPARCNEGOCIO = AGR.NRPARCNEGOCIO_REL
                        WHERE 
                          AGR.NRPARCNEGOCIO = :NRPARCNEGOCIO
                          AND AGR.CRM_NRORGTRAB = :CRM_NRORGTRAB
                          AND AGR.NRORG = :NRORG";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);

            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp;
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function searchParcnegocio($paramsQuery)
    {
        try {
            $query = "SELECT
                        PARC.NRPARCNEGOCIO,
                        PARC.NMPRINCIPALPARC,
                        PARC.NRINSCRICAOPARC
                      FROM
                        PARCNEGOCIO PARC
                        LEFT JOIN CRM_STATUSPARC STAT
                          ON STAT.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                            AND STAT.NRORG = PARC.NRORG
                            AND STAT.CRM_NRORGTRAB = :CRM_NRORGTRAB
                      WHERE 
                        PARC.NRPARCNEGOCIO = :NRPARCNEGOCIO
                        AND PARC.NRORG = :NRORG
                        AND STAT.STATUSPARC <> 'CLIENTE'";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);

            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp[0];
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getRelationshipPartnerStatus($paramsQuery)
    {
        try {
            $query = "SELECT
                        NRPARCNEGOCIO,
                        NMPRINCIPALPARC,
                        NRINSCRICAOPARC
                      FROM
                        PARCNEGOCIO PARC
                      WHERE 
                        NRPARCNEGOCIO = :NRPARCNEGOCIO
                        AND AGR.NRORG = :NRORG";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
            $resp = $std->fetchAll();

            if (!empty($resp)) {
                return $resp[0];
            } else {
                return array();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function setRelationshipPartnerStatus($paramsQuery)
    {
        try {
            $query = "UPDATE CRM_STATUSPARC
                      SET STATUSPARC = :STATUSPARC
                      WHERE NRORG = :NRORG
                        AND CRM_NRORGTRAB = :CRM_NRORGTRAB
                        AND NRPARCNEGOCIO = :NRPARCNEGOCIO";

            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findNewFilterCriteriaFactory($paramsQuery, $limit = 40, $page = 1)
    {
        try {

            if (session_status() == PHP_SESSION_NONE) {
                session_start();
            }

            if (!isset($_SESSION['current_page'])) {
                $_SESSION['current_page'] = 1;
            }

            // $page = $_SESSION['current_page'];
            $offset = ($page - 1) * $limit;

            $queryBuilder  = $this->connection->createQueryBuilder();

            $queryBuilder->select('PARCEIRO.INSCRICAO, PARCEIRO.NRPARCNEGOCIO, PARCEIRO.NOMEPRINCIPAL,
            PARCEIRO.NOMEFANTASIA, PARCEIRO.IDATIVO, PARCEIRO.TIPOPARCEIRO,
            PARCEIRO.NMOPERADOR, PARCEIRO.NRPARCRESPONSAVEL, PARCEIRO.NRSEQCONTATO,
            PARCEIRO.RESULTULTEVOL, PARCEIRO.MOTIVULTEVOL, PARCEIRO.NMTIPOCONT,
            PARCEIRO.NMMOTIEVOCON, PARCEIRO.DTHREVOLCONT, PARCEIRO.DIASULTEVOL, 
            PARCEIRO.DIASULTATIVCONC, PARCEIRO.CDPAIS, PARCEIRO.NMPAIS, 
            PARCEIRO.SGESTADO, PARCEIRO.NMESTADO, PARCEIRO.CDMUNICIPIO, 
            PARCEIRO.NMMUNICIPIO, PARCEIRO.CDBAIRRO, PARCEIRO.NMBAIRRO,
            PARCEIRO.NRORG, PARCEIRO.CRM_NRORGTRAB, PARCEIRO.DESCRESULEVOL,
            PARCEIRO.CDSEGMER, PARCEIRO.DSSEGMER, PARCEIRO.POSSUITAREFAS,
            PARCEIRO.POSSUIEVOLUCOES, PARCEIRO.IDSENEVOCONT')
                ->from('(' . $this->getSubquery() . ')', 'PARCEIRO')->distinct()
                ->orderBy('PARCEIRO.DTHREVOLCONT', 'DESC');

            if (isset($paramsQuery[0]['CDSEGMER']) && is_array($paramsQuery[0]['CDSEGMER'])) {
                $this->applyCdsegmerFilter($queryBuilder, $paramsQuery[0]['CDSEGMER']);
            }

            if (isset($paramsQuery[0]['INSCRICAO'])) {
                $this->applyInscriptionFilter($queryBuilder, $paramsQuery[0]['INSCRICAO']);
            }

            if (isset($paramsQuery[0]['NOMEPRINCIPAL'])) {
                $this->applyLegalNameFilter($queryBuilder, $paramsQuery[0]['NOMEPRINCIPAL']);
            }

            if (isset($paramsQuery[0]['NOMEFANTASIA'])) {
                $this->applyDoingBusinessFilter($queryBuilder, $paramsQuery[0]['NOMEFANTASIA']);
            }

            if (isset($paramsQuery[0]['IDATIVO'])) {
                $this->applyIdAtivo($queryBuilder, $paramsQuery[0]['IDATIVO']);
            }

            if (isset($paramsQuery[0]['TIPOPARCEIRO'])) {
                $this->applyPartnerTypeFilter($queryBuilder, $paramsQuery[0]['TIPOPARCEIRO']);
            }

            if (isset($paramsQuery[0]['STATUS'])) {
                $this->applyPartnerStatusFilter($queryBuilder, $paramsQuery[0]['STATUS']);
            }

            if (isset($paramsQuery[0]['NRSEQRESULEVOL']) && is_array($paramsQuery[0]['NRSEQRESULEVOL'])) {
                $this->applyResultOfLastEvolutionFilter($queryBuilder, $paramsQuery[0]['NRSEQRESULEVOL']);
            }

            if (isset($paramsQuery[0]['CDMOTIEVOCON']) && is_array($paramsQuery[0]['CDMOTIEVOCON'])) {
                $this->applyLastEvolutionReasonFilter($queryBuilder, $paramsQuery[0]['CDMOTIEVOCON']);
            }

            if (isset($paramsQuery[0]['CDPAIS'])) {
                $this->applyCountryFilter($queryBuilder, $paramsQuery[0]['CDPAIS']);
            }

            if (isset($paramsQuery[0]['SGESTADO'])) {
                $this->applyStateFilter($queryBuilder, $paramsQuery[0]['SGESTADO']);
            }

            if (isset($paramsQuery[0]['CDMUNICIPIO'])) {
                $this->applyCityFilter($queryBuilder, $paramsQuery[0]['CDMUNICIPIO']);
            }

            if (isset($paramsQuery[0]['CDBAIRRO'])) {
                $this->applyNeighborhoodFilter($queryBuilder, $paramsQuery[0]['CDBAIRRO']);
            }

            if (isset($paramsQuery[0]['POSSUIEVOLUCOES'])) {
                $this->applyhasEvolutionsFilter($queryBuilder, $paramsQuery[0]['POSSUIEVOLUCOES']);
            }

            if (isset($paramsQuery[0]['POSSUITAREFAS'])) {
                $this->applyhasTaskFilter($queryBuilder, $paramsQuery[0]['POSSUITAREFAS']);
            }

            if (isset($paramsQuery[0]['DIASULTEVOL'])) {
                $this->applyDaysWithoutEvolutionFilter($queryBuilder, $paramsQuery[0]['DIASULTEVOL']);
            }

            if (isset($paramsQuery[0]['DIASULTATIVCONC'])) {
                $this->applyNoTasksDoneFilter($queryBuilder, $paramsQuery[0]['DIASULTATIVCONC']);
            }

            if ((isset($paramsQuery[0]['DTINICIAL'])) && (isset($paramsQuery[0]['DTFIM']))) {
                $this->applyDateRangeFilter($queryBuilder, $paramsQuery[0]['DTINICIAL'], $paramsQuery[0]['DTFIM']);
            }

            if (isset($paramsQuery[0]['NRPARCRESPONSAVEL']) && is_array($paramsQuery[0]['NRPARCRESPONSAVEL'])) {
                $this->applyOperRespFilter($queryBuilder, $paramsQuery[0]['NRPARCRESPONSAVEL']);
            }


            $stmt = $queryBuilder->setFirstResult($offset)->setMaxResults($limit)->execute();
            $results = $stmt->fetchAll();
            $_SESSION['current_page']++;
            return $results;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getSubquery()
    {
        $subquery = "WITH CTE_PARC AS (
                SELECT 
                    NRINSCRICAOPARC AS INSCRICAO,
                    CDOPERRESPONSAVEL,
                    NRPARCNEGOCIO,
                    NMPRINCIPALPARC AS NOMEPRINCIPAL,
                    NMSECUNDARIPARC AS NOMEFANTASIA,
                    NRORG,
                    CDTIPOPARCPRINCIPAL AS TIPOPARCEIRO,
                    CDSEGMER
                FROM PARCNEGOCIO
                WHERE CDTIPOPARCPRINCIPAL NOT IN ('PESSOA', 'OPERADOR', 'REVENDA', 'INTERLOCUTOR', 'FORNECEDOR', 'ESTRUTURA', 'FUNCIONARIO')
            ),
            CTE_EVOL AS (
                SELECT
                    EVOL.DTHREVOLCONT,
                    EVOL.NRCONTATO,
                    EVOL.NRSEQCONTATO,
                    EVOL.NRSEQRESULEVOL AS RESULTULTEVOL,
                    EVOL.CDOPEREVOL,
                    EVOL.CDTIPOCONT,
                    EVOL.CDMOTIEVOCON AS MOTIVULTEVOL,  
                    EVOL.CRM_NRORGTRAB,
                    EVOL.IDSENEVOCONT,
                    EVOL.NRSEQFILACONTA
                FROM EVOLCONT EVOL
            ),
            CTE_MAIN AS (
                SELECT 
                    PARC.INSCRICAO,
                    PARC.NRPARCNEGOCIO,
                    PARC.NOMEPRINCIPAL,
                    PARC.NOMEFANTASIA,
                    PARC.NRORG,
                    PARC.TIPOPARCEIRO,
                    PARC.CDSEGMER,
                    EVOL.DTHREVOLCONT,
                    EVOL.NRCONTATO,
                    EVOL.NRSEQCONTATO,
                    EVOL.RESULTULTEVOL,
                    EVOL.CDOPEREVOL,
                    EVOL.CDTIPOCONT,
                    EVOL.MOTIVULTEVOL,
                    EVOL.CRM_NRORGTRAB,
                    EVOL.IDSENEVOCONT,
                    PARC.CDOPERRESPONSAVEL,
                    EVOL.NRSEQFILACONTA,        
                    CASE WHEN EVOL.DTHREVOLCONT IS NULL THEN 'N' ELSE 'S' END AS POSSUIEVOLUCOES,
                    TRUNC(SYSDATE - EVOL.DTHREVOLCONT) AS DIASULTEVOL,
                    ROW_NUMBER() OVER (PARTITION BY PARC.NRPARCNEGOCIO ORDER BY EVOL.DTHREVOLCONT DESC) AS RN
                FROM CTE_PARC PARC
                LEFT JOIN CTE_EVOL EVOL ON EVOL.NRSEQCONTATO = PARC.NRORG
            ),
            CTE_CRM_STATUSPARC AS (
                    SELECT 
                        NRPARCRESP AS NRPARCRESPONSAVEL,
                        IDATIVO,
                        NRPARCRESP,
                        NRPARCNEGOCIO,
                        NRORG, 
                        CDOPERRESP
                    FROM CRM_STATUSPARC
            ),
            CTE_OPERADOR AS (
                    SELECT
                        IDTPOPERADOR,
                        NMOPERADOR,
                        CDOPERADOR,
                        NRORG
                    FROM OPERADOR
            ),
            CTE_SEGMERCADO AS (
                    SELECT 
                        NRORG,
                        CDSEGMER,
                        DSSEGMER
                    FROM SEGMERCADO
            ),
            CTE_FILACONTATO AS (
                    SELECT 
                        NRSEQFILACONTA,
                        NRPARCNEGOCIO,
                        NRSEQCONTATO,
                        NRORG,
                        TRUNC(SYSDATE - DTREALFILACONTA) AS DIASULTATIVCONC,
                        CASE WHEN NRSEQFILACONTA IS NULL THEN 'N' ELSE 'S' END AS POSSUITAREFAS
                    FROM FILACONTATO
            ),
            CTE_TIPOCONT AS (
                    SELECT 
                        CDTIPOCONT,
                        NRORG,
                        NMTIPOCONT
                    FROM TIPOCONT
            ),
            CTE_MOTIEVOLCONT AS (
                    SELECT 
                        NMMOTIEVOCON,
                        CDMOTIEVOCON,
                        NRORG
                    FROM MOTIEVOLCONT
            ),
            CTE_CRM_RESULTEVOL AS (
                    SELECT
                        NRSEQRESULEVOL,
                        DESCRESULEVOL,
                        NRORG
                    FROM CRM_RESULTEVOL
            ),
            CTE_ENDERECOPARC AS (
                    SELECT 
                        NRORG,
                        NRPARCNEGOCIO,
                        CDPAIS,
                        SGESTADO,
                        CDMUNICIPIO,
                        CDBAIRRO
                    FROM ENDERECOPARC
            ),
            CTE_PAIS AS (
                    SELECT 
                        NRORG,
                        CDPAIS,
                        NMPAIS
                    FROM PAIS
            ),
            CTE_ESTADO AS (
                    SELECT 
                        NRORG,
                        CDPAIS,
                        SGESTADO,
                        NMESTADO
                    FROM ESTADO
            ),
            CTE_MUNICIPIO AS (
                    SELECT 
                        NRORG,
                        SGESTADO,
                        CDMUNICIPIO,
                        NMMUNICIPIO
                    FROM MUNICIPIO
            ),
            CTE_BAIRRO AS (
                    SELECT 
                        NRORG,
                        CDMUNICIPIO,
                        CDBAIRRO,
                        NMBAIRRO
                    FROM BAIRRO
            )
            SELECT
                    MAIN.INSCRICAO,
                    OPER.NMOPERADOR,
                    OPER.IDTPOPERADOR,
                    STAT.NRPARCRESPONSAVEL,
                    MAIN.NRPARCNEGOCIO,
                    MAIN.NOMEPRINCIPAL,
                    MAIN.NOMEFANTASIA,
                    MAIN.DTHREVOLCONT,
                    MAIN.NRCONTATO,
                    MAIN.NRSEQCONTATO,
                    MAIN.RESULTULTEVOL,
                    MAIN.NRORG,
                    MAIN.TIPOPARCEIRO,
                    MAIN.CDSEGMER,
                    MAIN.CDOPEREVOL,
                    MAIN.CDTIPOCONT,
                    MAIN.MOTIVULTEVOL,  
                    MAIN.CRM_NRORGTRAB,
                    TPCONT.NMTIPOCONT,
                    MTECONT.NMMOTIEVOCON,
                    RESEVOL.DESCRESULEVOL,
                    MAIN.IDSENEVOCONT,
                    ENDEPARC.CDPAIS,
                    ENDEPARC.SGESTADO,
                    ENDEPARC.CDMUNICIPIO,
                    ENDEPARC.CDBAIRRO,
                    PAIS.NMPAIS,
                    FILA.POSSUITAREFAS,
                    MAIN.POSSUIEVOLUCOES,
                    EST.NMESTADO,
                    MUNI.NMMUNICIPIO,
                    BAIRRO.NMBAIRRO,
                    FILA.DIASULTATIVCONC,
                    SEG.DSSEGMER,
                    STAT.IDATIVO,
                    STAT.NRPARCRESP,
                    MAIN.DIASULTEVOL
            FROM (SELECT * FROM CTE_MAIN WHERE RN = 1) MAIN
            LEFT JOIN CTE_CRM_STATUSPARC STAT ON MAIN.NRPARCNEGOCIO = STAT.NRPARCNEGOCIO AND STAT.NRORG = MAIN.NRORG
            LEFT JOIN CTE_OPERADOR OPER ON OPER.CDOPERADOR = MAIN.CDOPERRESPONSAVEL AND OPER.CDOPERADOR = STAT.CDOPERRESP AND OPER.NRORG = MAIN.NRORG
            LEFT JOIN CTE_SEGMERCADO SEG ON SEG.CDSEGMER = MAIN.CDSEGMER AND SEG.NRORG = MAIN.NRORG
            LEFT JOIN CTE_FILACONTATO FILA ON FILA.NRSEQFILACONTA = MAIN.NRSEQFILACONTA AND FILA.NRPARCNEGOCIO = MAIN.NRPARCNEGOCIO AND FILA.NRSEQCONTATO = MAIN.NRSEQCONTATO AND FILA.NRORG = MAIN.NRORG
            LEFT JOIN CTE_TIPOCONT TPCONT ON TPCONT.CDTIPOCONT = MAIN.CDTIPOCONT AND TPCONT.NRORG = MAIN.NRORG 
            LEFT JOIN CTE_MOTIEVOLCONT MTECONT ON MTECONT.CDMOTIEVOCON = MAIN.MOTIVULTEVOL AND MTECONT.NRORG = MAIN.NRORG
            LEFT JOIN CTE_CRM_RESULTEVOL RESEVOL ON RESEVOL.NRSEQRESULEVOL = MAIN.RESULTULTEVOL AND RESEVOL.NRORG = MAIN.NRORG
            LEFT JOIN CTE_ENDERECOPARC ENDEPARC ON ENDEPARC.NRPARCNEGOCIO = MAIN.NRPARCNEGOCIO AND ENDEPARC.NRORG = MAIN.NRORG
            LEFT JOIN CTE_PAIS PAIS ON PAIS.CDPAIS = ENDEPARC.CDPAIS AND PAIS.NRORG = ENDEPARC.NRORG
            LEFT JOIN CTE_ESTADO EST ON EST.CDPAIS = PAIS.CDPAIS AND EST.SGESTADO = ENDEPARC.SGESTADO AND EST.NRORG = ENDEPARC.NRORG
            LEFT JOIN CTE_MUNICIPIO MUNI ON MUNI.SGESTADO = EST.SGESTADO AND MUNI.CDMUNICIPIO = ENDEPARC.CDMUNICIPIO AND MUNI.NRORG = ENDEPARC.NRORG
            LEFT JOIN CTE_BAIRRO BAIRRO ON BAIRRO.CDMUNICIPIO = MUNI.CDMUNICIPIO AND BAIRRO.CDBAIRRO = ENDEPARC.CDBAIRRO AND BAIRRO.NRORG = ENDEPARC.NRORG";

        return $subquery;
    }


    private function applyInscriptionFilter($qb, $inscription)
    {
        $qb->andWhere($qb->expr()->like('PARCEIRO.INSCRICAO', ':inscription'))
            ->setParameter('inscription', '%' . $inscription . '%');
    }

    private function applyLegalNameFilter($qb, $legalName)
    {
        $qb->andWhere($qb->expr()->like('UPPER(PARCEIRO.NOMEPRINCIPAL)', 'UPPER(:legalName)'))
            ->setParameter('legalName', '%' . $legalName . '%');
    }

    private function applyDoingBusinessFilter($qb, $doingBusiness)
    {
        $qb->andWhere($qb->expr()->like('UPPER(PARCEIRO.NOMEFANTASIA)', 'UPPER(:doingBusiness)'))
            ->setParameter('doingBusiness', '%' . $doingBusiness . '%');
    }

    private function applyCdsegmerFilter($qb, $cdsegmer)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.CDSEGMER', ':cdsegmer'))
            ->setParameter('cdsegmer', $cdsegmer, Connection::PARAM_STR_ARRAY);
    }

    private function applyDateRangeFilter($qb, $startDate, $endDate)
    {
        $qb->andWhere(
            $qb->expr()->andX(
                $qb->expr()->gte('PARCEIRO.DTHREVOLCONT', 'TO_DATE(:startDate, \'YYYY-MM-DD\')'),
                $qb->expr()->lte('PARCEIRO.DTHREVOLCONT', 'TO_DATE(:endDate, \'YYYY-MM-DD\')')
            )
        )
            ->setParameter('startDate', $startDate)
            ->setParameter('endDate', $endDate);
    }

    private function applyDaysWithoutEvolutionFilter($qb, $daysWithoutUpdate)
    {
        $qb->andWhere('PARCEIRO.DIASULTEVOL >= :daysWithoutUpdate')
            ->setParameter('daysWithoutUpdate', $daysWithoutUpdate);
    }

    private function applyNoTasksDoneFilter($qb, $daysWithoutTasksDone)
    {
        $qb->andWhere('PARCEIRO.DIASULTATIVCONC >= :daysWithoutTasksDone')
            ->setParameter('daysWithoutTasksDone', $daysWithoutTasksDone);
    }

    private function applyResultOfLastEvolutionFilter($qb, $resultOfLastEvolution)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.RESULTULTEVOL', ':resultOfLastEvolution'))
            ->setParameter('resultOfLastEvolution', $resultOfLastEvolution, Connection::PARAM_STR_ARRAY);
    }

    private function applyLastEvolutionReasonFilter($qb, $evolutionReason)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.MOTIVULTEVOL', ':evolutionReason'))
            ->setParameter('evolutionReason', $evolutionReason, Connection::PARAM_STR_ARRAY);
    }

    private function applyIdAtivo($qb, $idAtivo)
    {
        $qb->andWhere($qb->expr()->eq('PARCEIRO.IDATIVO', ':idAtivo'))
            ->setParameter('idAtivo', $idAtivo);
    }

    private function applyPartnerTypeFilter($qb, $partnerType)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.TIPOPARCEIRO', ':partnerType'))
            ->setParameter('partnerType', $partnerType, Connection::PARAM_STR_ARRAY);
    }

    private function applyPartnerStatusFilter($qb, $statusPartner)
    {
        $qb->andWhere($qb->expr()->eq('PARCEIRO.STATUS', ':statusPartner'))
            ->setParameter('statusPartner', $statusPartner);
    }

    private function applyCountryFilter($qb, $country)
    {
        $qb->andWhere($qb->expr()->eq('PARCEIRO.CDPAIS', ':country'))
            ->setParameter('country', $country);
    }

    private function applyStateFilter($qb, $state)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.SGESTADO', ':state'))
            ->setParameter('state', $state);
    }

    private function applyCityFilter($qb, $city)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.CDMUNICIPIO', ':city'))
            ->setParameter('city', $city);
    }

    private function applyhasEvolutionsFilter($qb, $hasEvolution)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.POSSUIEVOLUCOES', ':hasEvolution'))
            ->setParameter('hasEvolution', $hasEvolution);
    }

    private function applyhasTaskFilter($qb, $hasTask)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.POSSUITAREFAS', ':hasTask'))
            ->setParameter('hasTask', $hasTask);
    }

    private function applyNeighborhoodFilter($qb, $neighborhood)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.CDBAIRRO', ':neighborhood'))
            ->setParameter('neighborhood', $neighborhood);
    }

    private function applyOperRespFilter($qb, $parcResp)
    {
        $qb->andWhere($qb->expr()->in('PARCEIRO.NRPARCRESPONSAVEL', ':parcResp'))
            ->setParameter('parcResp', $parcResp, Connection::PARAM_STR_ARRAY);
    }
}
